# -*- coding: utf-8 -*-
###############################################################################
#    Mobile Map Downloader - Download maps for your mobile phone.             #
#                                                                             #
#    Copyright (C) 2014 by Eike Welk                                          #
#    eike.welk@gmx.net                                                        #
#                                                                             #
#    License: GPL Version 3                                                   #
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.    #
###############################################################################
"""
Test the download functions.
"""

from __future__ import division
from __future__ import absolute_import              

#For test modules: ----------------------------------------------------------
#import pytest #contains `skip`, `fail`, `raises`, `config`

import time
import os
import os.path as path
from pprint import pprint
import urllib2


#Set up logging fore useful debug output, and time stamps in UTC.
import logging
logging.basicConfig(format='%(asctime)s: %(levelname)s: %(message)s', 
                    level=logging.DEBUG)
#Time stamps must be in UTC
logging.Formatter.converter = time.gmtime


def relative_path(*path_comps):
    "Create file paths that are relative to the location of this file."
    return path.abspath(path.join(path.dirname(__file__), *path_comps))

def find_index(list_like, search_val, key=lambda x:x):
    """
    Find the index of an element in a list like container. 
    
    Accepts a custom function to compute the comparison key, like ``list.sort``.
    
    Returns
    --------
    
    int | NoneType: Index of found element or ``None`` if no element is found.
    
    TODO: Write unit test.
    """
    for i, elem in enumerate(list_like):
        comp_val = key(elem)
        if comp_val == search_val:
            return i 
    else:
        return None
    
    
def test_BaseDownloader_download_file():
    "Test class OsmandDownloader: Downloading of files from Osmand server."
    from mob_map_dl.download import BaseDownloader
    
    print "Start download_file"
    test_map_name = relative_path("../../test_tmp/test_1.obf.zip")
    try: os.remove(test_map_name)
    except OSError: pass
    assert not path.exists(test_map_name)
    
    #File size 0.2 MiB
    url = "http://download.osmand.net/download.php?standard=yes&file=Monaco_europe_2.obf.zip"
#    #File size 3.0 MiB
#    url = "http://download.osmand.net/download.php?standard=yes&file=Jamaica_centralamerica_2.obf.zip"
    
    d = BaseDownloader()
    d.download_file(url, test_map_name, "test-file-name.foo")
    
    #Test name and size of downloaded file
    assert path.isfile(test_map_name)
    file_size = path.getsize(test_map_name)/1024**2
    print "file size [MiB]:", file_size
    assert round(file_size, 1) == 0.2
    
    
def test_OsmandDownloader_get_file_list():
    "Test class OsmandDownloader: Listing of files that can be downloaded."
    from mob_map_dl.download import OsmandDownloader
    
    print "Start get_file_list"
    d = OsmandDownloader()
    l = d.get_file_list()
    
    pprint(l)
    print len(l)
    
    #Test if some files exist
    get_disp_name = lambda e: e.disp_name
    assert find_index(l, "osmand/France_rhone-alpes_europe_2.obf", 
                      key=get_disp_name) is not None
    assert find_index(l, "osmand/Germany_nordrhein-westfalen_europe_2.obf", 
                      key=get_disp_name) is not None
    assert find_index(l, "osmand/Jamaica_centralamerica_2.obf", 
                      key=get_disp_name) is not None
    assert find_index(l, "osmand/Monaco_europe_2.obf", 
                      key=get_disp_name) is not None
    #Test the names of first & last file
    assert l[0].disp_name == "osmand/Afghanistan_asia_2.obf"
    assert l[-1].disp_name == "osmand/Zimbabwe_africa_2.obf"
    #Number of files must be in certain range.
    assert 500 < len(l) < 550
        
    #Test if the extracted URLs are correct
    idx = find_index(l, "osmand/Monaco_europe_2.obf", get_disp_name)
    url = l[idx].full_name
    fsrv = urllib2.urlopen(url)
    map_zip = fsrv.read()
    len_mib = len(map_zip) / 1024**2
    print len(map_zip), "B", len_mib, "MiB", round(len_mib, 1), "MiB rounded"
    assert round(len_mib, 1) == 0.2

    
def test_OpenandromapsDownloader_make_disp_name():
    from mob_map_dl.download import OpenandromapsDownloader
    
    print "Start"
    down = OpenandromapsDownloader()

    u1 = "http://www.openandromaps.org/maps/Germany/Germany_Mid_hike.zip"
    u2 = "http://www.openandromaps.org/maps/europe/France_North.zip"
    n1 = "oam/Germany_Germany_Mid_hike" 
    n2 = "oam/europe_France_North"
    
    n = down.make_disp_name(u1)
    assert n == n1
    
    n = down.make_disp_name(u2)
    assert n == n2


def test_OpenandromapsDownloader_get_file_list():
    from mob_map_dl.download import OpenandromapsDownloader
    
    print "Start"
    down = OpenandromapsDownloader()

    l = down.get_file_list()
    
    pprint(l)
    print len(l)
    
    #Number of files must be in certain range.
    assert 200 < len(l) < 250
    #Test if some files exist
    get_disp_name = lambda e: e.disp_name
    assert find_index(l, "oam/europe_France_South", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/Germany_Germany_Mid_hike", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/Russia_Central", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/usa_California", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/canada_Saskatchewan", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/SouthAmerica_Mexico", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/oceania_NewZealand", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/asia_Kazakhstan", 
                      key=get_disp_name) is not None
    assert find_index(l, "oam/africa_kenya", 
                      key=get_disp_name) is not None
    
    #Test if the extracted URLs are correct, (2 MiB download)
    idx = find_index(l, "oam/SouthAmerica_bermuda", get_disp_name)
    url = l[idx].full_name
    fsrv = urllib2.urlopen(url)
    map_zip = fsrv.read()
    len_mib = len(map_zip) / 1024**2
    print len(map_zip), "B", len_mib, "MiB", round(len_mib, 1), "MiB rounded"
    #Test rough dimensions, exact size changes with every new revision
    assert 1.8 < len_mib < 2.2


if __name__ == "__main__":
#    test_BaseDownloader_download_file()
#    test_OsmandDownloader_get_file_list()
#    test_OpenandromapsDownloader_make_disp_name()
#    test_OpenandromapsDownloader_get_file_list()
    
    pass #IGNORE:W0107
